import xml.etree.ElementTree as ET
import xml.dom.minidom
import math

file_name = input("Enter compressed KNML file path: ")

tree = ET.parse(file_name)
root = tree.getroot()
last_rev = ""
count = 0
length = len(root[0].findall('Instance'))

if length == 1:
	print("No revisions found, generate revisions from xmltoknml.py first")
	exit()

revisionsDict = {}

for each in root.iter('Instance'):
	instanceId = int(each.attrib['Id'])
	for child in each:
		if 'Body' in child.tag:
			revisionsDict[instanceId] = child[0].text


# n = int(input(str(length)+" Revisons found, enter the revision number to be loaded: "))
# original = n

for n in range(1, length+1):
	original = n
	m = int((math.log(length)) ** 2)+1;
	if n%m != 0:
		interval = n - (n%m) + 1
		n = n - interval + 1
	else:
		interval = n - (m-1)
		n = n - interval + 1


	count = interval
	prev_str = revisionsDict[count]
	result = prev_str
	while count < original:
		count += 1
		s = [x.replace("\n", "`").replace("-", "^") for x in prev_str.split(" ")]
		i = 0
		while(True):
			if i == len(s):
				break;
			if s[i].isspace() or s[i] == '':
				del s[i]
			else:	
				i += 1	

		next_rev = revisionsDict[count]
		s2 = next_rev.split(" ")
		i = 0
		while(True):
			if i == len(s2):
				break;
			if s2[i].isspace() or s2[i] == '':
				del s2[i]
			else:	
				i += 1	

		index = 0
		result = ""
		for x in s2:
			if x.isdigit():
				for i in range(index, index+int(x)):
		#			print(s[i], end=" ")
					result += s[i].replace("`", "\n").replace("^", "-")
					result += " "
					index += 1
			elif x[0] == "'" and x[-1] == "'" and x[1:-1].isdigit():
		#			print(x[1:-1].replace("`", "\n").replace("~", "-"), end=" ")				 
					result += x[1:-1].replace("`", "\n			").replace("^", "-")
					result += " "
			else:
				if x[0] == '-':
					for i in range(index, index+int(x[1:])):
						index += 1
				else:
		#			print(x.replace("`", "\n").replace("~", "-"), end=" ")
					result += x.replace("`", "\n			").replace("^", "-")		
					result += " "

		prev_str = result

	for each in root[0].findall('Instance')[original-1]:
		if 'Body' in each.tag:
			each[0].text = result

tree.write('output.knml')
print("All", original, "revisions written to output.knml")

f = open('output.knml')
f_str = f.read()
f.close()

f2 = open('output.knml', "w")
f2.write("<?xml version='1.0' encoding='utf-8'?>\n"+f_str)
f2.close()


from compressor import compress
from wiki_converter import wiki_knml_converter
import os

file_name = input("Enter path of XML file:")
wiki_knml_converter(file_name)
file_name = file_name[:-4] + '.knml'
compress(file_name)
os.remove(file_name)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec 13 11:18:54 2018

@author: descentis
"""

import xml.etree.cElementTree as ET

def getPostType(name):
    # To get an iterable
    n1 = name.split('/')
    postFile = n1[0]+'/Posts.xml'
    context_post = ET.iterparse(postFile, events=("start", "end"))
    # turning it into an iterator
    context_post = iter(context_post)
    
    # get the root element
    event_posts, root_posts = next(context_post)
    
    postType = {}
    for event, elem in context_post:
        if event == "end" and elem.tag == "row":
            li = []
            if(elem.attrib.get('ParentId')!=None):
                
                li.append(elem.attrib['PostTypeId'])
                li.append(elem.attrib['ParentId'])
                postType[elem.attrib['Id']] = li
            else:
                li.append(elem.attrib['PostTypeId'])
                postType[elem.attrib['Id']] = li
       
            elem.clear()     
            root_posts.clear()
    
    return postType


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct  7 12:55:50 2019

@author: descentis
"""

import xml.etree.ElementTree as ET
import xml.dom.minidom
import re
import difflib
import collections
from datetime import datetime

author_contribution = collections.defaultdict(lambda: [])

#Main Function
#file_name = input("Enter compressed KNML file path: ")
file_name = "2006_Westchester_County_torna.knolml"
d1 = datetime.strptime(input("Enter start date as YYYY-MM-DD format: "), '%Y-%m-%d')
d2 = datetime.strptime(input("Enter start date as YYYY-MM-DD format: "), '%Y-%m-%d')


tree = ET.parse(file_name)
root = tree.getroot()
last_rev = ""
count = 0
length = len(root[0].findall('Instance'))

revisionsList = []

last_contribution = 0

for each in root.iter('Instance'):
	instanceId = int(each.attrib['Id'])
	dict_key = 0
	dict_val = 0
	for child in each:
		if 'TimeStamp' in child.tag:
			contr_date = datetime.strptime(child[0].text.split("T")[0], '%Y-%m-%d')
			if d1 > contr_date or d2 < contr_date:
				break 
		if 'Contributors' in child.tag:
			dict_key = revision = child[0].text
		if 'Body' in child.tag:
			dict_val = len(child[0].text.split())
			#print(dict_key)
			#print(dict_val)	
			author_contribution[dict_key].append(dict_val-last_contribution)
			last_contribution = dict_val
#List of all revisions			
#print(revisionsList)

#Sentence Number, modified sentence
for i in range(1, len(revisionsList)):
	analyze(revisionsList[i-1].split(), revisionsList[i].split(), i+1)


print("\n\n======Authors' Contribution==========\n\n")
'''
for x in sorted(author_contribution.items() ,  key=lambda x: x[1]):
	print(x[0], "\n--->", x[1])
'''
for x in author_contribution.items():
	print(x[0], "\n--->", x[1])


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun  7 06:54:41 2018

@author: descentis
"""

import requests
import io



def get_wiki_byname(featuredArticleList):
	# articleName = raw_input()
	# articleName = articleName.replace(' ', '_')

	for each in featuredArticleList:
		articleName = each

		file_handler = io.open(articleName+'.xml', mode='w+', encoding='utf-8')

		url = 'https://en.m.wikipedia.org/w/index.php?title=Special:Export&pages=' + articleName + '&history=1&action=submit'
		headers = {
			'user-agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.139 Mobile Safari/537.36'
		}
		print('Downloading ' + articleName + '...') 
		r = requests.get(url, headers=headers)
		if r.status_code == 200:
			xml = r.text
			file_handler.write(xml)
			print(articleName,'Completed!')
		else:
			print('Something went wrong! ' + articleName + '\n' + '\n')

		file_handler.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 11 21:48:53 2018

@author: descentis
"""

import xml.etree.cElementTree as ET
import textwrap
import html

'''
This function is used to indent the xml; document is pretty style
'''

instance_id = 1

def indent(elem, level=0):
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i

def wiki_file_writer(elem,myFile,prefix):
    global instance_id
    t = '\t'

    Instance = t+t+"<Instance "
    
  
    for ch_elem in elem:        
                    
        if(('id' in ch_elem.tag) and ('parentid' not in ch_elem.tag)):             
            Instance = Instance+ "Id="+'"'+str(instance_id)+'"'+" InstanceType="+'"'+"Revision/Wiki"+'"'+" RevisionId="+ '"'+str(ch_elem.text)+'"'+">\n"
            myFile.write(Instance)
            
            '''
            RevisionId = t+t+t+"<RevisionId>"+ch_elem.text+"</RevisionId>\n"
            myFile.write(RevisionId)
            '''
        
        '''
        if(ch_elem.tag==prefix+'parentid'):
            ParentId = t+t+t+"<ParentId>"+ch_elem.text+"</ParentId>\n" 
            myFile.write(ParentId)
        '''
            

            
        
                  
        
        '''
        Timestamp Information
        '''
        if('timestamp' in ch_elem.tag):
            '''
            if(f_p!=1):
                Instance = Instance+" InstanceType= "+'"'+"wiki/text"+'"'+">\n"
                myFile.write(Instance)
            '''
            Timestamp = t+t+t+"<TimeStamp>\n"
            myFile.write(Timestamp)
            CreationDate = t+t+t+t+"<CreationDate>"+ch_elem.text[:-1]+'.0'+"</CreationDate>\n"
            myFile.write(CreationDate)
            Timestamp = t+t+t+"</TimeStamp>\n"
            myFile.write(Timestamp)            
            
            
        '''
        Contributors information
        '''
        if('contributor' in ch_elem.tag):            
            Contributors = t+t+t+"<Contributors>\n"
            myFile.write(Contributors)
            for contrib in ch_elem:
                if('ip' in contrib.tag):
                    LastEditorUserName = t+t+t+t+"<OwnerUserName>"+html.escape(contrib.text)+"</OwnerUserName>\n"
                    myFile.write(LastEditorUserName)                        
                else:
                    if('username' in contrib.tag):
                        LastEditorUserName = t+t+t+t+"<OwnerUserName>"+html.escape(contrib.text)+"</OwnerUserName>\n"
                        myFile.write(LastEditorUserName)                        
                    if(('id' in contrib.tag) and ('parentid' not in contrib.tag)):
                        LastEditorUserId = t+t+t+t+"<OwnerUserId>"+contrib.text+"</OwnerUserId>\n"
                        myFile.write(LastEditorUserId)
                
                    
            Contributors = t+t+t+"</Contributors>\n"
            myFile.write(Contributors)
        
        
        '''
        Body/Text Information
        '''
        if('text' in ch_elem.tag):
            Body = t+t+t+"<Body>\n"
            myFile.write(Body)
            if(ch_elem.attrib.get('bytes')!=None):
                text_field = t+t+t+t+"<Text Type="+'"'+"wiki/text"+'"'+" Bytes="+'"'+ch_elem.attrib['bytes']+'">\n'
            elif(ch_elem.text != None):
                text_field = t+t+t+t+"<Text Type="+'"'+"wiki/text"+'"'+" Bytes="+'"'+str(len(ch_elem.text))+'">\n'
            else:
                text_field = t+t+t+t+"<Text Type="+'"'+"wiki/text"+'"'+" Bytes="+'"'+str(0)+'">\n'
            myFile.write(text_field)
            if(ch_elem.text == None):                
                text_body = "";
            else:
               
                text_body = textwrap.indent(text=ch_elem.text, prefix=t+t+t+t+t)
                text_body = html.escape(text_body)
            Body_text = text_body+"\n"
            myFile.write(Body_text)
            text_field = t+t+t+t+"</Text>\n"
            myFile.write(text_field)        
            Body = t+t+t+"</Body>\n"
            myFile.write(Body)            
        

        
        if('comment' in ch_elem.tag):
            Edit = t+t+t+"<EditDetails>\n"
            myFile.write(Edit)
            if(ch_elem.text == None):                
                text_body = "";
            else:
                text_body = textwrap.indent(text=ch_elem.text, prefix=t+t+t+t+t)
                text_body = html.escape(text_body)
            
            EditType = t+t+t+t+"<EditType>\n"+text_body+"\n"+t+t+t+t+"</EditType>\n"
            #Body_text = text_body+"\n"
            myFile.write(EditType)
            
            Edit = t+t+t+"</EditDetails>\n"
            myFile.write(Edit)    

        if('sha1' in ch_elem.tag):
            sha = ch_elem.text
            if(type(sha)!=type(None)):
                shaText = t+t+t+'<data key="sha">'+sha+'</data>\n'
                myFile.write(shaText)
            else:
                shaText = ''
            
    Instance = t+t+"</Instance>\n"
    myFile.write(Instance)  
    instance_id+=1             

def wiki_knml_converter(name):
    global instance_id
    #Creating a meta file for the wiki article
    
    
    
    # To get an iterable for wiki file
    file_name = name
    context_wiki = ET.iterparse(file_name, events=("start","end"))
    # Turning it into an iterator
    context_wiki = iter(context_wiki)
    
    # getting the root element
    event_wiki, root_wiki = next(context_wiki)
    file_name = name[:-4]+'.knolml'
    file_path = file_name
    with open(file_path,"w",encoding='utf-8') as myFile:
        myFile.write("<?xml version='1.0' encoding='utf-8'?>\n")
        myFile.write("<KnolML>\n")
        myFile.write('<key attr.name="sha" attrib.type="string" for="Instance" id="sha"/>\n')
       
    prefix = '{http://www.mediawiki.org/xml/export-0.10/}'    #In case of Wikipedia, prefic is required
    f = 0
    title_text = ''
    for event, elem in context_wiki:
        
        if event == "end" and 'id' in elem.tag:
            if(f==0):
                with open(file_path,"a",encoding='utf-8') as myFile:
                     myFile.write("\t<KnowledgeData "+"Type="+'"'+"Wiki/text/revision"+'"'+" Id="+'"'+elem.text+'"'+">\n")
                     
                f=1
                    
        if event == "end" and 'title' in elem.tag:
            title_text = elem.text

        if(f==1 and title_text!=None):            
            Title = "\t\t<Title>"+title_text+"</Title>\n"
            with open(file_path,"a",encoding='utf-8') as myFile:
                myFile.write(Title)
            title_text = None
        if event == "end" and 'revision' in elem.tag:
     
            with open(file_path,"a",encoding='utf-8') as myFile:
                wiki_file_writer(elem,myFile,prefix)
                
                
            elem.clear()
            root_wiki.clear() 

    with open(file_path,"a",encoding='utf-8') as myFile:
        myFile.write("\t</KnowledgeData>\n")
        myFile.write("</KnolML>\n") 

    instance_id = 1


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Mar  2 18:56:24 2019

@author: descentis
"""

import os
from multiprocessing import Process, Lock
import time
import numpy as np
import glob
import difflib
import xml.etree.ElementTree as ET
import math
import textwrap
import html
import requests
import io

class wikiConverter(object):


    instance_id = 1
    
    
    def indent(self,elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.indent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i
    
    @staticmethod
    def wiki_file_writer(elem,myFile,prefix):
        global instance_id
        t = '\t'
    
        Instance = t+t+"<Instance "
        
      
        for ch_elem in elem:        
                        
            if(('id' in ch_elem.tag) and ('parentid' not in ch_elem.tag)):             
                Instance = Instance+ "Id="+'"'+str(wikiConverter.instance_id)+'"'+" InstanceType="+'"'+"Revision/Wiki"+'"'+" RevisionId="+ '"'+str(ch_elem.text)+'"'+">\n"
                myFile.write(Instance)
                
                '''
                RevisionId = t+t+t+"<RevisionId>"+ch_elem.text+"</RevisionId>\n"
                myFile.write(RevisionId)
                '''
            
            '''
            if(ch_elem.tag==prefix+'parentid'):
                ParentId = t+t+t+"<ParentId>"+ch_elem.text+"</ParentId>\n" 
                myFile.write(ParentId)
            '''
                
    
                
            
                      
            
            '''
            Timestamp Information
            '''
            if('timestamp' in ch_elem.tag):
                '''
                if(f_p!=1):
                    Instance = Instance+" InstanceType= "+'"'+"wiki/text"+'"'+">\n"
                    myFile.write(Instance)
                '''
                Timestamp = t+t+t+"<TimeStamp>\n"
                myFile.write(Timestamp)
                CreationDate = t+t+t+t+"<CreationDate>"+ch_elem.text[:-1]+'.0'+"</CreationDate>\n"
                myFile.write(CreationDate)
                Timestamp = t+t+t+"</TimeStamp>\n"
                myFile.write(Timestamp)            
                
                
            '''
            Contributors information
            '''
            if('contributor' in ch_elem.tag):            
                Contributors = t+t+t+"<Contributors>\n"
                myFile.write(Contributors)
                for contrib in ch_elem:
                    if('ip' in contrib.tag):
                        LastEditorUserName = t+t+t+t+"<OwnerUserName>"+html.escape(contrib.text)+"</OwnerUserName>\n"
                        myFile.write(LastEditorUserName)                        
                    else:
                        if('username' in contrib.tag):
                            try:
                                LastEditorUserName = t+t+t+t+"<OwnerUserName>"+html.escape(contrib.text)+"</OwnerUserName>\n"
                            except:
                                LastEditorUserName = t+t+t+t+"<OwnerUserName>None</OwnerUserName>\n"
                            myFile.write(LastEditorUserName)                        
                        if(('id' in contrib.tag) and ('parentid' not in contrib.tag)):
                            LastEditorUserId = t+t+t+t+"<OwnerUserId>"+contrib.text+"</OwnerUserId>\n"
                            myFile.write(LastEditorUserId)
                    
                        
                Contributors = t+t+t+"</Contributors>\n"
                myFile.write(Contributors)
            
            
            '''
            Body/Text Information
            '''
            if('text' in ch_elem.tag):
                Body = t+t+t+"<Body>\n"
                myFile.write(Body)
                if(ch_elem.attrib.get('bytes')!=None):
                    text_field = t+t+t+t+"<Text Type="+'"'+"wiki/text"+'"'+" Bytes="+'"'+ch_elem.attrib['bytes']+'">\n'
                elif(ch_elem.text != None):
                    text_field = t+t+t+t+"<Text Type="+'"'+"wiki/text"+'"'+" Bytes="+'"'+str(len(ch_elem.text))+'">\n'
                else:
                    text_field = t+t+t+t+"<Text Type="+'"'+"wiki/text"+'"'+" Bytes="+'"'+str(0)+'">\n'
                myFile.write(text_field)
                if(ch_elem.text == None):                
                    text_body = "";
                else:
                   
                    text_body = textwrap.indent(text=ch_elem.text, prefix=t+t+t+t+t)
                    text_body = html.escape(text_body)
                Body_text = text_body+"\n"
                myFile.write(Body_text)
                text_field = t+t+t+t+"</Text>\n"
                myFile.write(text_field)        
                Body = t+t+t+"</Body>\n"
                myFile.write(Body)            
            
    
            
            if('comment' in ch_elem.tag):
                Edit = t+t+t+"<EditDetails>\n"
                myFile.write(Edit)
                if(ch_elem.text == None):                
                    text_body = "";
                else:
                    text_body = textwrap.indent(text=ch_elem.text, prefix=t+t+t+t+t)
                    text_body = html.escape(text_body)
                
                EditType = t+t+t+t+"<EditType>\n"+text_body+"\n"+t+t+t+t+"</EditType>\n"
                #Body_text = text_body+"\n"
                myFile.write(EditType)
                
                Edit = t+t+t+"</EditDetails>\n"
                myFile.write(Edit)    
    
            if('sha1' in ch_elem.tag):
                sha = ch_elem.text
                if(type(sha)!=type(None)):
                    shaText = t+t+t+'<Knowl key="sha">'+sha+'</Knowl>\n'
                    myFile.write(shaText)
                else:
                    shaText = ''
                
        Instance = t+t+"</Instance>\n"
        myFile.write(Instance)  
        wikiConverter.instance_id+=1             
 
    @staticmethod
    def wiki_knolml_converter(name, *args, **kwargs):
        #global instance_id
        #Creating a meta file for the wiki article
        
        
        
        # To get an iterable for wiki file

        file_name = name
        context_wiki = ET.iterparse(file_name, events=("start","end"))
        # Turning it into an iterator
        context_wiki = iter(context_wiki)
        
        # getting the root element
        event_wiki, root_wiki = next(context_wiki)
        file_name = name[:-4]+'.knolml'
        file_path = file_name
        if kwargs.get('output_dir'):
            file_path = file_path.replace('output','wikipedia_articles')
            
        with open(file_path,"w",encoding='utf-8') as myFile:
            myFile.write("<?xml version='1.0' encoding='utf-8'?>\n")
            myFile.write("<KnolML>\n")
            myFile.write('<Def attr.name="sha" attrib.type="string" for="Instance" id="sha"/>\n')
           
        prefix = '{http://www.mediawiki.org/xml/export-0.10/}'    #In case of Wikipedia, prefic is required
        f = 0
        title_text = ''
        for event, elem in context_wiki:
            
            if event == "end" and 'id' in elem.tag:
                if(f==0):
                    with open(file_path,"a",encoding='utf-8') as myFile:
                         myFile.write("\t<KnowledgeData "+"Type="+'"'+"Wiki/text/revision"+'"'+" Id="+'"'+elem.text+'"'+">\n")
                         
                    f=1
                        
            if event == "end" and 'title' in elem.tag:
                title_text = elem.text
    
            if(f==1 and title_text!=None):            
                Title = "\t\t<Title>"+title_text+"</Title>\n"
                with open(file_path,"a",encoding='utf-8') as myFile:
                    myFile.write(Title)
                title_text = None
            if event == "end" and 'revision' in elem.tag:
         
                with open(file_path,"a",encoding='utf-8') as myFile:
                    wikiConverter.wiki_file_writer(elem,myFile,prefix)
                    
                    
                elem.clear()
                root_wiki.clear() 
    
        with open(file_path,"a",encoding='utf-8') as myFile:
            myFile.write("\t</KnowledgeData>\n")
            myFile.write("</KnolML>\n") 
    
        wikiConverter.instance_id = 1


    @staticmethod
    def is_number(s):
        try:
            int(s)
            return True
        except ValueError:
            return False
    
    @staticmethod
    def encode(str1, str2):
    	output = ""
    	s = [x.replace("\n", "`").replace("-", "^") for x in str1.split(" ")]
    
    	s2 = [x.replace("\n", "`").replace("-", "^") for x in str2.split(" ")]
    
    	i = 0
    	while(True):
    		if i == len(s):
    			break;
    		if s[i].isspace() or s[i] == '':
    			del s[i]
    		else:	
    			i += 1	
    	i = 0
    	while(True):
    		if i == len(s2):
    			break;
    		if s2[i].isspace() or s2[i] == '':
    			del s2[i]
    		else:	
    			i += 1	
    			
    	d = difflib.Differ()
    
    	result = list(d.compare(s, s2))
    
    	pos = 0
    	neg = 0
    
    	for x in result:
    		if x[0] == " ":
    			pos += 1
    			if neg != 0:
    				output += "-"+str(neg)+" "
    				neg = 0
    		elif x[0] == "-":
    			neg += 1
    			if pos != 0:
    				output += str(pos)+" "
    				pos = 0	
    		elif x[0] != "?":
    			if pos != 0:
    				output += str(pos)+" "
    				pos = 0	
    			if neg != 0:
    				output += "-"+str(neg)+" "
    				neg = 0
    			if wikiConverter.is_number(x[2:]):
    				output += "'"+x[2:]+"' "
    			else:			
    				output += x[2:]+" "
    	if pos != 0:
    		output += str(pos)+" "
    	if neg != 0:
    		output += "-"+str(neg)+" "
    	return output.replace("\t\t\t", "")
    
    #Main function
    
    @staticmethod
    def compress(file_name, directory):
    	# file_name = input("Enter path of KML file:")
    
        tree = ET.parse(file_name)
        r = tree.getroot()
        for child in r:
            if('KnowledgeData' in child.tag):
                child.attrib['Type'] = 'Wiki/text/revision/compressed'
                root = child
                
        last_rev = ""
        length = len(root.findall('Instance'))
    
        print(length, "revisions found")
    
        count = 0
        intervalLength =  int((math.log(length)) ** 2);  
    
        # Keep the Orginal text after every 'm' revisions
        m = intervalLength+1
        for each in root.iter('Text'):
            count += 1
            if m != intervalLength+1:
                current_str = each.text
                each.text = wikiConverter.encode(prev_str, current_str)
                prev_str = current_str
                # print("Revision ", count, " written")
    			
                m = m - 1
                if m == 0:
                    m = intervalLength+1
            else:
                prev_str = each.text
                # print("Revision ", count, " written")
                m = m - 1
                continue
    
        print("KnolML file created")
    	
        # Creating directory 
        if not os.path.exists(directory):
            os.mkdir(directory)
    
        # Changing file path to include directory
        file_name = file_name.split('/')
        file_name = directory+'/'+file_name[-1]
        '''
        file_name.insert(-1, directory)
        separator = '/'
        file_name = separator.join(file_name)
        '''
    
        tree.write(file_name[:-7]+'.knolml')
        f = open(file_name[:-7]+'.knolml')
        f_str = f.read()
        f.close()
    
        f2 = open(file_name[:-7]+'.knolml', "w")
        f2.write("<?xml version='1.0' encoding='utf-8'?>\n"+f_str)
        f2.close()
    
    
    @staticmethod
    def wikiConvert(*args, **kwargs):

        if(kwargs.get('output_dir')!=None):
            output_dir = kwargs['output_dir']        
        if(kwargs.get('file_name')!=None):
            file_name = kwargs['file_name']
            wikiConverter.wiki_knolml_converter(file_name)
            file_name = file_name[:-4] + '.knolml'
            wikiConverter.compress(file_name,output_dir)
            os.remove(file_name)            
       
        if(kwargs.get('file_list')!=None):
            path_list = kwargs['file_list']
            for file_name in path_list:            
                wikiConverter.wiki_knolml_converter(file_name)
                file_name = file_name[:-4] + '.knolml'
                wikiConverter.compress(file_name,output_dir)
                os.remove(file_name)
        
        if((kwargs.get('file_name')==None) and (kwargs.get('file_list')==None)):
            print("No arguments provided")
    
    def returnList(self, l, n):
        for i in range(0,len(l),n):
            yield l[i:i+n]

    
     
    @staticmethod
    def compressAll(dir_path, *args, **kwargs):
        t1 = time.time()
        if(kwargs.get('c_num')!=None):
            c_num = kwargs['c_num']
        else:
            c_num = 4              # By default it is 4
        fileNames = glob.glob(dir_path+'/*.xml')
        if(kwargs.get('output_dir')!=None):
            output_dir=kwargs['output_dir']
        else:
            output_dir = os.getcwd()
        fileNum = len(fileNames)
        fileList = []
        if(fileNum<c_num):
            for f in fileNames:
                fileList.append([f])
        
        else:           

            f = np.array_split(fileNames,c_num)
            for i in f:
                fileList.append(i.tolist())
        
        l = Lock()
        processDict = {}
        if(fileNum<c_num):
            pNum = fileNum
        else:
            pNum = c_num
        for i in range(pNum):
            processDict[i+1] = Process(target=wikiConverter.wikiConvert,kwargs={'output_dir':output_dir,'file_list': fileList[i],'l': l})
        
        for i in range(pNum):
            processDict[i+1].start()
        
        for i in range(pNum):
            processDict[i+1].join()
        
        
        t2 = time.time()
        
        print("All process done with time: ",str(t2-t1))

    @staticmethod
    def convertwiki(*args, **kwargs):

        if(kwargs.get('output_dir')!=None):
            output_dir = kwargs['output_dir']        
        if(kwargs.get('file_name')!=None):
            file_name = kwargs['file_name']
            wikiConverter.wiki_knolml_converter(file_name,output_dir=output_dir)
            file_name = file_name[:-4] + '.knolml'
            #wikiConverter.compress(file_name,output_dir)
            #os.remove(file_name)            
       
        if(kwargs.get('file_list')!=None):
            path_list = kwargs['file_list']
            for file_name in path_list:            
                wikiConverter.wiki_knolml_converter(file_name, output_dir=output_dir)
                file_name = file_name[:-4] + '.knolml'
                #wikiConverter.compress(file_name,output_dir)
                #os.remove(file_name)
        
        if((kwargs.get('file_name')==None) and (kwargs.get('file_list')==None)):
            print("No arguments provided")
            

    @staticmethod
    def convertall(dir_path, *args, **kwargs):
        t1 = time.time()
        if(kwargs.get('c_num')!=None):
            c_num = kwargs['c_num']
        else:
            c_num = 4              # By default it is 4
        fileNames = glob.glob(dir_path+'/*.xml')
        if(kwargs.get('output_dir')!=None):
            output_dir=kwargs['output_dir']
        else:
            output_dir = os.getcwd()
        fileNum = len(fileNames)
        fileList = []
        if(fileNum<c_num):
            for f in fileNames:
                fileList.append([f])
        
        else:           

            f = np.array_split(fileNames,c_num)
            for i in f:
                fileList.append(i.tolist())
        
        l = Lock()
        processDict = {}
        if(fileNum<c_num):
            pNum = fileNum
        else:
            pNum = c_num
        for i in range(pNum):
            processDict[i+1] = Process(target=wikiConverter.convertwiki,kwargs={'output_dir':output_dir,'file_list': fileList[i],'l': l})
        
        for i in range(pNum):
            processDict[i+1].start()
        
        for i in range(pNum):
            processDict[i+1].join()
        
        
        t2 = time.time()
        
        print("All process done with time: ",str(t2-t1))        
        
        
    @staticmethod    
    def getArticle(*args, **kwargs):
    	# articleName = raw_input()
    	# articleName = articleName.replace(' ', '_')
        featuredArticleList = []
        if(kwargs.get('file_name')!=None):
            featuredArticleList.append(kwargs['file_name'])            
       
        if(kwargs.get('file_list')!=None):
            featuredArticleList = kwargs['file_list']
            
        if(kwargs.get('output_dir')!=None):
            output_dir = kwargs['output_dir']+'/'
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
        else:
            output_dir = ''
    
        for each in featuredArticleList:
            articleName = each
            articleName = articleName.replace(' ','_')
            articleName = articleName.replace('/','__')
            file_handler = io.open(output_dir+articleName+'.xml', mode='w+', encoding='utf-8')
            url = 'https://en.m.wikipedia.org/w/index.php?title=Special:Export&pages=' + articleName + '&history=1&action=submit'
            headers = {
			'user-agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.139 Mobile Safari/537.36'
            }
            print('Downloading ' + articleName + '...') 
            r = requests.get(url, headers=headers)
            if r.status_code == 200:
                xml = r.text
                file_handler.write(xml)
                print(articleName,'Completed!')
            else:
                print('Something went wrong! ' + articleName + '\n' + '\n')
            
            file_handler.close()
            wikiConverter.wiki_knolml_converter(output_dir+articleName+'.xml')    
    
    @staticmethod    
    def serialCompress(self,dir_path, *args, **kwargs):
        t1 = time.time()
        file_list = os.listdir(dir_path)
        path_list = []
        
        if(kwargs.get('output_dir')!=None):
            output_dir=kwargs['output_dir']
        else:
            output_dir = os.getcwd()
        for f in file_list:
            path_list.append(dir_path+'/'+f)
        
        self.convert(path_list,output_dir=output_dir)
        t2 = time.time()
        
        print("all process done: ",str(t2-t1))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 11:01:20 2019

@author: descentis
"""

import sqlite3
import ast

conn = sqlite3.connect('/home/descentis/research/KML/nidhi_database/articleDescdb.db')																			#connecting to database  
print("Connection made")
#conn.execute("drop table article_desc;")                                                                           #dropped the existing table with same name
#conn.execute("create table article_desc(article_nm text,vital_article text,level text,class text,topic text);");   #cretaed table article_desc
#print("table article_desc created")
#conn.execute("create table wiki_project(article_nm text,project text;")                                            #created table wiki_projet
#print("table wiki_project created")


def display_data(query):
    cursor = conn.execute(query)
    displayList = []
    for row in cursor:
         displayList.append(row)
    return displayList

def insert_article_data(csvfile):
    line_no = 0                                                                                      #variable to count number of lines
    for line in open(csvfile, encoding='latin-1'):                                                   #opens csv file and loop to fetch each line in file
        line_no = line_no + 1
        if line_no==1:
            continue
        else:
            query = "insert into article_desc(article_nm,vital_article,level,class,topic) values("   #string variable 'query' to store mysql query, creating insert query
            arr = line.split('@$@')                                                                  #spliting the file line fetched based on the deliminator '@$@'
            for i in range(0, 5):                                                                    #loop to fetch each value obtained after spliting the line
                if i == 0:
                    query = query + '"' + arr[i] + '"'
                else:
                    query = query + ',"' + arr[i] + '"'
            query = query + ");"
            print(line_no)
            conn.execute(query)                                                                      #executing query                                                                              #saving changes by every 1000 lines in database
    print("all data inserted")
    conn.commit()

def insert_wiki_project(csvfile):
    line_no = 0                                                                                      #variable to count number of lines
    for line in open(csvfile, encoding='latin-1'):                                                   #opens csv file, and loop to fetch each line in file
        line_no = line_no + 1
        if line_no==1 or line_no==3:
            continue
        else:
            arr=line.split('@$@')                                                                           #splites the value in the csv file based on the deliminator"@$@"
            title = arr[0]
            temp = arr[5]                                                                                   #accessing the last value in the line , wikipedia projects, they are in the form an array

            if temp.startswith('NA'):                                                                       #if wikipedia project name not available
                    query = "insert into wiki_project(article_nm,project) values('" + title + "',NULL);"    #creating insert query
                    conn.execute(query)                                                                     #executing insert query
            else:                                                                                           #if wikipedia project name avaialble
                    arr2=ast.literal_eval(temp)                                                             #converting it into array
                    if arr2==[]:                                                                            #if array is empty
                        query = "insert into wiki_project(article_nm,project) values('"+ title + "',NULL);" #creating insert query
                        conn.execute(query)                                                                 #executing insert query
                    else:                                                                                   #if array contains values
                        for val in arr2:
                            vali=val.replace("'","\\'")
                            query="insert into wiki_project(article_nm,project) values('"+title+"','"+vali+"');" #creating insert query
                            conn.execute(query)                                                                  #executing insert query
            print(line_no)                                                                                       #save changes to database after executing every 1000 lines
    print("all data inserted")
    conn.commit()                                                                                                # saving the changes in the database




#insert_article_data('articleDesc.csv')                                          #this function call insert the table article_desc with article  data like(article name,vital article,level,topic ,class)
#print('Data inserted in articledesc')

#insert_wiki_project('articleDesc.csv')                                          #this function call would insert all the data in table wiki_project with article data like (article name , and project category)
#print("Data inserted in wiki_project")

'''
print("First 10 values in wiki_project")
display_data("select *from wiki_project limit 10;")                              #will displays first 10 values present in the table wiki_project

print("\nFirst 10 values in article_desc")
display_data("select *from article_desc limit 10;")                              #will display first 10 values present in table article_desc

print("Operation performed successfully")
'''

#faList = display_data("select article_nm from article_desc where class = 'B';")
#faList = display_data("select article_id,article_nm,class from article_desc where article_nm = 'Lionel Messi';")
#faList = display_data("select article_nm,project from wiki_project where article_nm = 'Lionel Messi';")
#faList = display_data("select article_id, article_nm from article_desc where class = 'FA';")
#k = display_data("select *from wiki_project where project='india';")
#football = display_data("select article_nm,project from wiki_project where project='football';")
#biography = display_data("select article_nm,project from wiki_project where project='biography';")
#countries = display_data("select article_nm,project from wiki_project where project='countries';")
faList = display_data("select article_nm from article_desc where class = 'FA';")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr  2 20:57:27 2019

@author: descentis

"""

from kdap.analysis import instances
from kdap.converter.qaConverter import qaConverter
from kdap.analysis import knol
#from kdap.converter.wikiConverter import wikiConverter
#import xml.etree.ElementTree as ET
#import glob
#import re


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 11 21:29:07 2018

@author: descentis
"""

import xml.etree.cElementTree as ET
import textwrap
import html
import os.path
from datetime import datetime
import glob
from subprocess import call
import errno
import os
from internetarchive import download

class qaConverter(object):
    
    def __init__(self):
        self.random = 'just to check'
        print(self.random)
    
    @staticmethod
    def getPostType(name):
        # To get an iterable
        n1 = name.split('/')
        postFile = n1[0]+'/Posts.xml'
        context_post = ET.iterparse(postFile, events=("start", "end"))
        # turning it into an iterator
        context_post = iter(context_post)
        
        # get the root element
        event_posts, root_posts = next(context_post)
        
        postType = {}
        for event, elem in context_post:
            if event == "end" and elem.tag == "row":
                li = []
                if(elem.attrib.get('ParentId')!=None):
                    
                    li.append(elem.attrib['PostTypeId'])
                    li.append(elem.attrib['ParentId'])
                    postType[elem.attrib['Id']] = li
                else:
                    li.append(elem.attrib['PostTypeId'])
                    postType[elem.attrib['Id']] = li
           
                elem.clear()     
                root_posts.clear()
        
        return postType    
    
    @staticmethod
    def make_path(path):
        try:
            os.makedirs(path)
        except OSError as exception:
            if exception.errno != errno.EEXIST:
                raise    

    @staticmethod
    def indent(elem, level=0):
        i = "\n" + level*"  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                qaConverter.indent(elem, level+1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i
    
    @staticmethod
    def tag_remover(body):
        for i in body:
            if(i=='<'):
                body = body.replace(i,"")
            if(i=='>'):
                body = body.replace(i,", ")
        return body
    
    
    @staticmethod
    def writeHistoryData(elem,post_id,instanceId,HistoryType,postType,val,name):
        filePath = name+'/'
        if(val==1 and postType[0]!='2'):
            filePath = filePath+"Post"+str(post_id)+".knolml"
            
            with open(filePath,"w",encoding='utf-8') as myFile:
                myFile.write("<?xml version='1.0' encoding='utf-8'?>\n")
                myFile.write("<KnolML>\n") 
                myFile.write("\t<KnowledgeData "+"Type="+'"'+"QA/text"+'"'+" Id= "+'"'+str(post_id)+'"'+">\n")
                t = '\t'
                f2 = 0
                if(postType[0]!='2'):
                    Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"InstanceType= "+'"Revision/Question" '+"RevisionId= "+ '"'+str(elem.attrib['RevisionGUID'])+'"'+">\n"
                    myFile.write(Instance)
                    f2 = 1
                elif(postType[0]=='2'):
                    Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"OriginalId= "+'"'+str(post_id)+'"'+" "+"InstanceType= "+'"Revision/Answer" '+"RevisionId= "+ '"'+str(elem.attrib['RevisionGUID'])+'"'+">\n"                
                    myFile.write(Instance)
                    f2=1
    
                
                
                if(f2==1):
        
                    '''
                    Timestamp information
                    '''
                
                    TimeStamp = t+t+t+"<TimeStamp>\n "
                    myFile.write(TimeStamp)
                    if(elem.attrib.get("CreationDate") != None):
                        CreationDate = t+t+t+t+"<CreationDate>"+elem.attrib['CreationDate']+"</CreationDate> \n"
                        myFile.write(CreationDate)
                    if(elem.attrib.get("LastEditDate") != None):
                        LastEditDate = t+t+t+t+"<LastEditDate>"+elem.attrib['LastEditDate']+"</LastEditDate> \n"
                        myFile.write(LastEditDate)
                    if(elem.attrib.get("LastActivityDate") != None):
                        LastActivityDate = t+t+t+t+"<LastActivityDate>"+elem.attrib['LastActivityDate']+"</LastActivityDate> \n"
                        myFile.write(LastActivityDate)
                    if(elem.attrib.get("CommunityOwnedDate") != None):
                        CommunityOwnedDate = t+t+t+t+"<CommunityOwnedDate>"+elem.attrib['CommunityOwnedDate']+"</CommunityOwnedDate> \n"
                        myFile.write(CommunityOwnedDate)
                    if(elem.attrib.get("ClosedDate") != None):
                        ClosedDate = t+t+t+t+"<ClosedDate>"+elem.attrib['ClosedDate']+"</ClosedDate> \n"
                        myFile.write(ClosedDate)
                    TimeStamp = t+t+t+"</TimeStamp>\n "
                    myFile.write(TimeStamp)
        
        
                    '''
                    Contributors information
                    '''
                    
                    
                    Contributors = t+t+t+"<Contributors>\n"
                    myFile.write(Contributors)
                    if(elem.attrib.get("UserId") != None):
                        Contributors =t+t+t+t+"<OwnerUserId>"+elem.attrib['UserId']+"</OwnerUserId>\n"
                        myFile.write(Contributors)
                    Contributors = t+t+t+"</Contributors>\n"
                    myFile.write(Contributors)


                    '''
                    PostHistory might have a comment associated with each entry.
                    I have added that section as editDetails
                    '''
        
                    Edit = t+t+t+"<EditDetails>\n"
                    myFile.write(Edit)
                    
                    if(HistoryType.get(int(elem.attrib['PostHistoryTypeId']))!=None):
                        EditType = t+t+t+t+"<EditType>"+HistoryType[int(elem.attrib['PostHistoryTypeId'])]+"</EditType>\n"
                    else:
                        EditType = t+t+t+t+"<EditType>Unknown</EditType>\n"
                    myFile.write(EditType)
        
                    if(elem.attrib.get('Comment')!=None):
                        text_body = textwrap.indent(text=elem.attrib['Comment'], prefix=t+t+t+t)
                        text_body = html.escape(text_body)
                        Body_text = text_body+"\n"
                        EditType = t+t+t+t+"<EditComment>"+Body_text+"</EditComment>\n"
                        myFile.write(EditType)
                    Edit = t+t+t+"</EditDetails>\n"
                    myFile.write(Edit)
                    
        
                    '''
                    Writing the body/text part
                    '''
                    if(elem.attrib.get('Text')!=None):        
                        Body = t+t+t+"<Body>\n"
                        myFile.write(Body)
                        text_field = t+t+t+t+"<Text Type= "+'"'+"text"+'"'+" Bytes="+'"'+str(len(elem.attrib['Text']))+'">\n'
                        myFile.write(text_field)
                        if(elem.attrib['PostHistoryTypeId']=="3" or elem.attrib['PostHistoryTypeId']=="6"):            
                            body_text = elem.attrib['Text']
                            body_text = qaConverter.tag_remover(body_text)
                            text_body = textwrap.indent(text=body_text, prefix=t+t+t+t+t)
                            text_body = html.escape(text_body)
                        else:
                            text_body = textwrap.indent(text=elem.attrib['Text'], prefix=t+t+t+t+t)
                            text_body = html.escape(text_body)
                        Body_text = text_body+"\n"
                        myFile.write(Body_text)
                        text_field = t+t+t+t+"</Text>\n"
                        myFile.write(text_field)        
                        Body = t+t+t+"</Body>\n"
                        myFile.write(Body)
        

                    
                    Instance = t+t+"</Instance>\n"
                    myFile.write(Instance)
                     
                    
        
        else:
            f = 0
            if(postType[0]!='2'):
                filePath = filePath+"Post"+str(post_id)+".knolml"
                f = 1
            elif(postType[0]=='2'):
                
                filePath = filePath+"Post"+str(postType[1])+".knolml"
                f = 1
    
            
            if(f==1):
                with open(filePath,"a",encoding='utf-8') as myFile:
                    t = '\t'
                    if(postType[0]=='1'):
                        Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"InstanceType= "+'"Revision/Question" '+"RevisionId= "+ '"'+str(elem.attrib['RevisionGUID'])+'"'+">\n"
                    else:
                        Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"OriginalId= "+'"'+str(post_id)+'"'+" "+"InstanceType= "+'"Revision/Answer" '+"RevisionId= "+ '"'+str(elem.attrib['RevisionGUID'])+'"'+">\n"
                    myFile.write(Instance)
                    
        
        
                    '''
                    Timestamp information
                    '''
                
                    TimeStamp = t+t+t+"<TimeStamp>\n "
                    myFile.write(TimeStamp)
                    if(elem.attrib.get("CreationDate") != None):
                        CreationDate = t+t+t+t+"<CreationDate>"+elem.attrib['CreationDate']+"</CreationDate> \n"
                        myFile.write(CreationDate)
                    if(elem.attrib.get("LastEditDate") != None):
                        LastEditDate = t+t+t+t+"<LastEditDate>"+elem.attrib['LastEditDate']+"</LastEditDate> \n"
                        myFile.write(LastEditDate)
                    if(elem.attrib.get("LastActivityDate") != None):
                        LastActivityDate = t+t+t+t+"<LastActivityDate>"+elem.attrib['LastActivityDate']+"</LastActivityDate> \n"
                        myFile.write(LastActivityDate)
                    if(elem.attrib.get("CommunityOwnedDate") != None):
                        CommunityOwnedDate = t+t+t+t+"<CommunityOwnedDate>"+elem.attrib['CommunityOwnedDate']+"</CommunityOwnedDate> \n"
                        myFile.write(CommunityOwnedDate)
                    if(elem.attrib.get("ClosedDate") != None):
                        ClosedDate = t+t+t+t+"<ClosedDate>"+elem.attrib['ClosedDate']+"</ClosedDate> \n"
                        myFile.write(ClosedDate)
                    TimeStamp = t+t+t+"</TimeStamp>\n "
                    myFile.write(TimeStamp)
        
        
                    '''
                    Contributors information
                    '''
                    
                    
    
                    if(elem.attrib.get("UserId") != None):
                        Contributors = t+t+t+"<Contributors>\n"
                        myFile.write(Contributors)
                        Contributors =t+t+t+t+"<OwnerUserId>"+elem.attrib['UserId']+"</OwnerUserId>\n"
                        myFile.write(Contributors)
                        Contributors = t+t+t+"</Contributors>\n"
                        myFile.write(Contributors)
                    
        
                    '''
                    Writing the body/text part
                    '''
                    if(elem.attrib.get('Text')!=None):        
                        Body = t+t+t+"<Body>\n"
                        myFile.write(Body)
                        text_field = t+t+t+t+"<Text Type= "+'"'+"text"+'"'+" Bytes="+'"'+str(len(elem.attrib['Text']))+'">\n'
                        myFile.write(text_field)
                        if(elem.attrib['PostHistoryTypeId']=="3" or elem.attrib['PostHistoryTypeId']=="6"):            
                            body_text = elem.attrib['Text']
                            body_text = qaConverter.tag_remover(body_text)
                            text_body = textwrap.indent(text=body_text, prefix=t+t+t+t+t)
                            text_body = html.escape(text_body)
                        else:
                            text_body = textwrap.indent(text=elem.attrib['Text'], prefix=t+t+t+t+t)
                            text_body = html.escape(text_body)
                        Body_text = text_body+"\n"
                        myFile.write(Body_text)
                        text_field = t+t+t+t+"</Text>\n"
                        myFile.write(text_field)        
                        Body = t+t+t+"</Body>\n"
                        myFile.write(Body)
        
                    '''
                    PostHistory might have a comment associated with each entry.
                    I have added that section as editDetails
                    '''
        
                    Edit = t+t+t+"<EditDetails>\n"
                    myFile.write(Edit)
                    
                    if(HistoryType.get(int(elem.attrib['PostHistoryTypeId']))!=None):
                        EditType = t+t+t+t+"<EditType>"+HistoryType[int(elem.attrib['PostHistoryTypeId'])]+"</EditType>\n"
                    else:
                        EditType = t+t+t+t+"<EditType>Unknown</EditType>\n"
                    myFile.write(EditType)
        
                    if(elem.attrib.get('Comment')!=None):
                        text_body = textwrap.indent(text=elem.attrib['Comment'], prefix=t+t+t+t+t)
                        text_body = html.escape(text_body)
                        Body_text = text_body+"\n"
                        EditType = t+t+t+t+"<EditComment>\n"
                        myFile.write(EditType)
                        myFile.write(Body_text)
                        EditType = t+t+t+t+"</EditComment>\n"
                        myFile.write(EditType)
                    Edit = t+t+t+"</EditDetails>\n"
                    myFile.write(Edit)        
    
                    Instance = t+t+"</Instance>\n"
                    myFile.write(Instance)
    
    
    @staticmethod
    def postHistoryConversion(name):
    
        # To get an iterable
        n1 = name.split('/')
        postHistoryFile = n1[0]+'/PostHistory.xml'
        context_postHistory = ET.iterparse(postHistoryFile, events=("start", "end"))
        # turning it into an iterator
        context_postHistory = iter(context_postHistory)
        
        # get the root element
        event_posts, root_posts = next(context_postHistory)
    
        # Dictionary of postHistoryType
        HistoryType = {1:'Initial Title',2:'Initial Body',3:'Initial Tags',4:'Edit Title',5:'Edit Body',6:'Edit Tags',7:'Rollback Title',8:'Rollback Body',
                       9:'Rollback Tags',10:'Post Closed',11:'Post Reopened',12:'Post Deleted',13:'Post Undeleted',14:'Post Locked',15:'Post Unlocked',
                       16:'Community Owned',17:'Post Migrated',18:'Question Merged',19:'Question Protected',20:'Question Unprotected',21:'Post Disassociated',
                       22:'Question Unmerged'}
        
        postIdBuffer = {}
        postType = qaConverter.getPostType(name)
        
        instanceId = 1
        for event, elem in context_postHistory:
            if event == "end" and elem.tag == "row":
                post_id = elem.attrib['PostId']
    
                if(postType[post_id][0]=='1'):
                    if(postIdBuffer.get(post_id)==None):
                        postIdBuffer[post_id] = 1
                        qaConverter.writeHistoryData(elem,post_id,postIdBuffer[post_id],HistoryType,postType[post_id],1,name)
                    else:
                        postIdBuffer[post_id]+=1
                        qaConverter.writeHistoryData(elem,post_id,postIdBuffer[post_id],HistoryType,postType[post_id],2,name)
                    
                    instanceId+=1
                elif(postType[post_id][0]=='2'):
                    if(postIdBuffer.get(post_id)==None):
                        if(postIdBuffer.get(postType[post_id][1])!=None):
                            postIdBuffer[postType[post_id][1]] += 1
                            qaConverter.writeHistoryData(elem,post_id,postIdBuffer[postType[post_id][1]],HistoryType,postType[post_id],1,name)
                        
                    else:
                        if(postIdBuffer.get(postType[post_id][1])!=None):
                            postIdBuffer[postType[post_id][1]]+=1
                            qaConverter.writeHistoryData(elem,post_id,postIdBuffer[postType[post_id][1]],HistoryType,postType[post_id],2,name)
                    instanceId+=1
                    
                else:
                    if(postIdBuffer.get(post_id)==None):
                        postIdBuffer[post_id] = 1
                        qaConverter.writeHistoryData(elem,post_id,postIdBuffer[post_id],HistoryType,postType[post_id],1,name)
                    else:
                        postIdBuffer[post_id]+=1
                        qaConverter.writeHistoryData(elem,post_id,postIdBuffer[post_id],HistoryType,postType[post_id],2,name)
                    
                    instanceId+=1                    

                elem.clear()
                root_posts.clear()
        

        '''
        file_list = glob.glob(name+"/*.knolml")
        for file_path in file_list:
            if(os.path.isfile(file_path)):
                with open(file_path,"a",encoding='utf-8') as myFile:
                    myFile.write("\t</KnowledgeData>\n")
                    myFile.write("</KnolML>\n")           
        
        '''
        return postType
        




    '''
    Writing the data for posts
    '''

    @staticmethod
    def writePostData(elem,post_id,instanceId,postType,val,name):
        filePath = name+'/'
        if(val==1 and postType[0]!='2'):
            filePath = filePath+"Post"+str(post_id)+".knolml"
            
            with open(filePath,"w",encoding='utf-8') as myFile:
                t = '\t'
                myFile.write("<?xml version='1.0' encoding='utf-8'?>\n")
                myFile.write("<KnolML>\n") 
                myFile.write("\t<KnowledgeData "+"Type="+'"'+"QA/text"+'"'+" Id= "+'"'+str(post_id)+'"'+">\n")
                if(elem.attrib.get('Title')!=None):
                    myFile.write(t+t+"<Title>"+html.escape(elem.attrib['Title'])+"</Title>\n")
                
                f2 = 0
                if(postType[0]!='2'):
                    Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"InstanceType= "+'"Question"'+">\n"
                    myFile.write(Instance)
                    f2 = 1
                elif(postType[0]=='2'):
                    Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"OriginalId= "+'"'+str(post_id)+'"'+" "+"InstanceType= "+'"Answer"'+">\n"                
                    myFile.write(Instance)
                    f2=1
    
                
                
                if(f2==1):
        
                    '''
                    Timestamp information
                    '''
                
                    TimeStamp = t+t+t+"<TimeStamp>\n "
                    myFile.write(TimeStamp)
                    if(elem.attrib.get("CreationDate") != None):
                        CreationDate = t+t+t+t+"<CreationDate>"+elem.attrib['CreationDate']+"</CreationDate> \n"
                        myFile.write(CreationDate)
                    if(elem.attrib.get("LastEditDate") != None):
                        LastEditDate = t+t+t+t+"<LastEditDate>"+elem.attrib['LastEditDate']+"</LastEditDate> \n"
                        myFile.write(LastEditDate)
                    if(elem.attrib.get("LastActivityDate") != None):
                        LastActivityDate = t+t+t+t+"<LastActivityDate>"+elem.attrib['LastActivityDate']+"</LastActivityDate> \n"
                        myFile.write(LastActivityDate)
                    if(elem.attrib.get("CommunityOwnedDate") != None):
                        CommunityOwnedDate = t+t+t+t+"<CommunityOwnedDate>"+elem.attrib['CommunityOwnedDate']+"</CommunityOwnedDate> \n"
                        myFile.write(CommunityOwnedDate)
                    if(elem.attrib.get("ClosedDate") != None):
                        ClosedDate = t+t+t+t+"<ClosedDate>"+elem.attrib['ClosedDate']+"</ClosedDate> \n"
                        myFile.write(ClosedDate)
                    TimeStamp = t+t+t+"</TimeStamp>\n "
                    myFile.write(TimeStamp)
        
        
                    '''
                    Contributors information
                    '''
                    
                    
                    Contributors = t+t+t+"<Contributors>\n"
                    myFile.write(Contributors)
                    if(elem.attrib.get("OwnerUserId") != None):
                        Contributors =t+t+t+t+"<OwnerUserId>"+elem.attrib['OwnerUserId']+"</OwnerUserId>\n"
                        myFile.write(Contributors)

                    if(elem.attrib.get("LastEditorUserId") != None):
                        Contributors =t+t+t+t+"<LastEditorUserId>"+elem.attrib['LastEditorUserId']+"</LastEditorUserId>\n"
                        myFile.write(Contributors)
                    Contributors = t+t+t+"</Contributors>\n"
                    myFile.write(Contributors)


                    '''
                    PostHistory might have a comment associated with each entry.
                    I have added that section as editDetails
                    
        
                    Edit = t+t+t+"<EditDetails>\n"
                    myFile.write(Edit)
                    
                    if(HistoryType.get(int(elem.attrib['PostHistoryTypeId']))!=None):
                        EditType = t+t+t+t+"<EditType>"+HistoryType[int(elem.attrib['PostHistoryTypeId'])]+"</EditType>\n"
                    else:
                        EditType = t+t+t+t+"<EditType>Unknown</EditType>\n"
                    myFile.write(EditType)
        
                    if(elem.attrib.get('Comment')!=None):
                        text_body = textwrap.indent(text=elem.attrib['Comment'], prefix=t+t+t+t)
                        text_body = html.escape(text_body)
                        Body_text = text_body+"\n"
                        EditType = t+t+t+t+"<EditComment>"+Body_text+"</EditComment>\n"
                        myFile.write(EditType)
                    Edit = t+t+t+"</EditDetails>\n"
                    myFile.write(Edit)
                    '''
                    
        
                    '''
                    Writing the body/text part
                    '''
                    if(elem.attrib.get('Body')!=None):        
                        Body = t+t+t+"<Body>\n"
                        myFile.write(Body)
                        text_field = t+t+t+t+"<Text Type= "+'"'+"text"+'"'+" Bytes="+'"'+str(len(elem.attrib['Body']))+'">\n'
                        myFile.write(text_field)
            
                        body_text = elem.attrib['Body']
                        #body_text = self.tag_remover(body_text)
                        text_body = html.escape(body_text)
                        text_body = textwrap.indent(text=text_body, prefix=t+t+t+t+t)
                        

                        Body_text = text_body+"\n"
                        myFile.write(Body_text)
                        text_field = t+t+t+t+"</Text>\n"
                        myFile.write(text_field)        
                        Body = t+t+t+"</Body>\n"
                        myFile.write(Body)

                    
                    if(elem.attrib.get('Tags')!=None):
                        Tags_element = t+t+t+"<Tags>"+html.escape(elem.attrib['Tags'])+"</Tags>\n"
                        
                        myFile.write(Tags_element)
                    
                    Reputation_tag = t+t+t+"<Credit> \n"
                    myFile.write(Reputation_tag)
                    if(elem.attrib.get("Score") != None):
                        score = t+t+t+t+"<Score>"+elem.attrib['Score']+"</Score>\n"
                        myFile.write(score)
                    if(elem.attrib.get("ViewCount") != None):
                        ViewCount = t+t+t+t+"<ViewCount>"+elem.attrib['ViewCount']+"</ViewCount>\n"
                        myFile.write(ViewCount)
                    if(elem.attrib.get("AnswerCount") != None):        
                        AnswerCount = t+t+t+t+"<AnswerCount>"+elem.attrib['AnswerCount']+"</AnswerCount>\n"
                        myFile.write(AnswerCount)
                    if(elem.attrib.get("CommentCount") != None):
                        CommentCount = t+t+t+t+"<CommentCount>"+elem.attrib['CommentCount']+"</CommentCount>\n"
                        myFile.write(CommentCount)
                
                    if(elem.attrib.get("FavouriteCount") != None):
                        FavouriteCount = t+t+t+t+"<FavouriteCount>"+elem.attrib['FavouriteCount']+"</FavouriteCount>\n"
                        myFile.write(FavouriteCount)
                
                    Reputation_tag = t+t+t+"</Credit> \n"
                    myFile.write(Reputation_tag)        

                    
                    Instance = t+t+"</Instance>\n"
                    myFile.write(Instance)
                     
                    
        
        else:
            f = 0
            if(postType[0]!='2'):
                filePath = filePath+"Post"+str(post_id)+".knolml"
                f = 1
            elif(postType[0]=='2'):
                
                filePath = filePath+"Post"+str(postType[1])+".knolml"
                f = 1
    
            
            if(f==1):
                with open(filePath,"a",encoding='utf-8') as myFile:
                    t = '\t'
                    if(postType[0]=='1'):
                        Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"InstanceType= "+'"Question"'+">\n"
                    else:
                        Instance = t+t+"<Instance Id= "+'"'+str(instanceId)+'"'+" "+"OriginalId= "+'"'+str(post_id)+'"'+" "+"InstanceType= "+'"Answer"'+">\n"
                    myFile.write(Instance)
                    
        
        
                    '''
                    Timestamp information
                    '''
                
                    TimeStamp = t+t+t+"<TimeStamp>\n "
                    myFile.write(TimeStamp)
                    if(elem.attrib.get("CreationDate") != None):
                        CreationDate = t+t+t+t+"<CreationDate>"+elem.attrib['CreationDate']+"</CreationDate> \n"
                        myFile.write(CreationDate)
                    if(elem.attrib.get("LastEditDate") != None):
                        LastEditDate = t+t+t+t+"<LastEditDate>"+elem.attrib['LastEditDate']+"</LastEditDate> \n"
                        myFile.write(LastEditDate)
                    if(elem.attrib.get("LastActivityDate") != None):
                        LastActivityDate = t+t+t+t+"<LastActivityDate>"+elem.attrib['LastActivityDate']+"</LastActivityDate> \n"
                        myFile.write(LastActivityDate)
                    if(elem.attrib.get("CommunityOwnedDate") != None):
                        CommunityOwnedDate = t+t+t+t+"<CommunityOwnedDate>"+elem.attrib['CommunityOwnedDate']+"</CommunityOwnedDate> \n"
                        myFile.write(CommunityOwnedDate)
                    if(elem.attrib.get("ClosedDate") != None):
                        ClosedDate = t+t+t+t+"<ClosedDate>"+elem.attrib['ClosedDate']+"</ClosedDate> \n"
                        myFile.write(ClosedDate)
                    TimeStamp = t+t+t+"</TimeStamp>\n "
                    myFile.write(TimeStamp)
        
        
                    '''
                    Contributors information
                    '''
                    
                    
    
                    Contributors = t+t+t+"<Contributors>\n"
                    myFile.write(Contributors)
                    if(elem.attrib.get("OwnerUserId") != None):
                        Contributors =t+t+t+t+"<OwnerUserId>"+elem.attrib['OwnerUserId']+"</OwnerUserId>\n"
                        myFile.write(Contributors)

                    if(elem.attrib.get("LastEditorUserId") != None):
                        Contributors =t+t+t+t+"<LastEditorUserId>"+elem.attrib['LastEditorUserId']+"</LastEditorUserId>\n"
                        myFile.write(Contributors)
                    Contributors = t+t+t+"</Contributors>\n"
                    myFile.write(Contributors)
                    
        
                    '''
                    Writing the body/text part
                    '''
                    if(elem.attrib.get('Body')!=None):        
                        Body = t+t+t+"<Body>\n"
                        myFile.write(Body)
                        text_field = t+t+t+t+"<Text Type= "+'"'+"text"+'"'+" Bytes="+'"'+str(len(elem.attrib['Body']))+'">\n'
                        myFile.write(text_field)
            
                        body_text = elem.attrib['Body']
                        #body_text = self.tag_remover(body_text)
                        text_body = html.escape(body_text)
                        text_body = textwrap.indent(text=text_body, prefix=t+t+t+t+t)
                        

                        Body_text = text_body+"\n"
                        myFile.write(Body_text)
                        text_field = t+t+t+t+"</Text>\n"
                        myFile.write(text_field)        
                        Body = t+t+t+"</Body>\n"
                        myFile.write(Body)


                    if(elem.attrib.get('Tags')!=None):
                        Tags_element = t+t+t+"<Tags>"+html.escape(elem.attrib['Tags'])+"</Tags>\n"
                        
                        myFile.write(Tags_element)
                    
                    Reputation_tag = t+t+t+"<Credit> \n"
                    myFile.write(Reputation_tag)
                    if(elem.attrib.get("Score") != None):
                        score = t+t+t+t+"<Score>"+elem.attrib['Score']+"</Score>\n"
                        myFile.write(score)
                    if(elem.attrib.get("ViewCount") != None):
                        ViewCount = t+t+t+t+"<ViewCount>"+elem.attrib['ViewCount']+"</ViewCount>\n"
                        myFile.write(ViewCount)
                    if(elem.attrib.get("AnswerCount") != None):        
                        AnswerCount = t+t+t+t+"<AnswerCount>"+elem.attrib['AnswerCount']+"</AnswerCount>\n"
                        myFile.write(AnswerCount)
                    if(elem.attrib.get("CommentCount") != None):
                        CommentCount = t+t+t+t+"<CommentCount>"+elem.attrib['CommentCount']+"</CommentCount>\n"
                        myFile.write(CommentCount)
                
                    if(elem.attrib.get("FavouriteCount") != None):
                        FavouriteCount = t+t+t+t+"<FavouriteCount>"+elem.attrib['FavouriteCount']+"</FavouriteCount>\n"
                        myFile.write(FavouriteCount)
                
                    Reputation_tag = t+t+t+"</Credit> \n"
                    myFile.write(Reputation_tag)        

        
                    '''
                    PostHistory might have a comment associated with each entry.
                    I have added that section as editDetails
                    
        
                    Edit = t+t+t+"<EditDetails>\n"
                    myFile.write(Edit)
                    
                    if(HistoryType.get(int(elem.attrib['PostHistoryTypeId']))!=None):
                        EditType = t+t+t+t+"<EditType>"+HistoryType[int(elem.attrib['PostHistoryTypeId'])]+"</EditType>\n"
                    else:
                        EditType = t+t+t+t+"<EditType>Unknown</EditType>\n"
                    myFile.write(EditType)
        
                    if(elem.attrib.get('Comment')!=None):
                        text_body = textwrap.indent(text=elem.attrib['Comment'], prefix=t+t+t+t+t)
                        text_body = html.escape(text_body)
                        Body_text = text_body+"\n"
                        EditType = t+t+t+t+"<EditComment>\n"
                        myFile.write(EditType)
                        myFile.write(Body_text)
                        EditType = t+t+t+t+"</EditComment>\n"
                        myFile.write(EditType)
                    Edit = t+t+t+"</EditDetails>\n"
                    myFile.write(Edit)
                    '''
    
                    Instance = t+t+"</Instance>\n"
                    myFile.write(Instance)





    '''
    This function converts the Post.xml file into KML
    '''
    
    @staticmethod
    def postConversion(name):
        
        # To get an iterable
        n1 = name.split('/')
        postFile = n1[0]+'/Posts.xml'
        context_post = ET.iterparse(postFile, events=("start", "end"))
        # turning it into an iterator
        context_post = iter(context_post)
        
        # get the root element
        event_posts, root_posts = next(context_post)
        
        postIdBuffer = {}
        #postType = getPostType()
        
        instanceId = 1
        for event, elem in context_post:
            if event == "end" and elem.tag == "row":
                post_id = elem.attrib['Id']
                
                if(elem.attrib['PostTypeId']=='1'):
                    postType = [elem.attrib['PostTypeId']]
                    if(postIdBuffer.get(post_id)==None):
                        postIdBuffer[post_id] = 1
                        qaConverter.writePostData(elem,post_id,postIdBuffer[post_id],postType,1,name)
                    else:
                        
                        postIdBuffer[post_id]+=1
                        qaConverter.writePostData(elem,post_id,postIdBuffer[post_id],postType,2,name)
                    
                    instanceId+=1
                elif(elem.attrib['PostTypeId']=='2'):
                    postType = [elem.attrib['PostTypeId'],elem.attrib['ParentId']]
                    if(postIdBuffer.get(post_id)==None):
                        if(postIdBuffer.get(postType[1])!=None):
                            postIdBuffer[postType[1]] += 1
                            qaConverter.writePostData(elem,post_id,postIdBuffer[postType[1]],postType,1,name)
                        
                    else:
                        if(postIdBuffer.get(postType[1])!=None):
                            postIdBuffer[postType[1]] += 1
                            qaConverter.writePostData(elem,post_id,postIdBuffer[postType[1]],postType,2,name)
                    instanceId+=1
                    
                else:
                    postType = [elem.attrib['PostTypeId']]
                    if(postIdBuffer.get(post_id)==None):
                        postIdBuffer[post_id] = 1
                        qaConverter.writePostData(elem,post_id,postIdBuffer[post_id],postType,1,name)
                    else:
                        
                        postIdBuffer[post_id]+=1
                        qaConverter.writePostData(elem,post_id,postIdBuffer[post_id],postType,2,name)
                    
                    instanceId+=1
                    
                    
                elem.clear()
                root_posts.clear()
        
        '''
        file_list = glob.glob(name+"/*.knolml")
        for file_path in file_list:
            if(os.path.isfile(file_path)):
                with open(file_path,"a",encoding='utf-8') as myFile:
                    myFile.write("</KnolML>\n") 
        '''
        return postType        

    @staticmethod
    def commentsConversion(name,p):
        if(p==0):
            postType = qaConverter.postHistoryConversion(name)
            #self.postConversion(name)
        else:
            postType = qaConverter.getPostType(name)
            qaConverter.postConversion(name)
            

        # To get an iterable
        n1 = name.split('/')
        commentFile = n1[0]+'/Comments.xml'
        context_comments = ET.iterparse(commentFile, events=("start", "end"))
        # turning it into an iterator
        context_comments = iter(context_comments)
        
        # get the root element
        event_posts, root_posts = next(context_comments)
        
        date_format = "%Y-%m-%dT%H:%M:%S.%f"
        for event,elem in context_comments:
            if event == "end" and elem.tag == "row":
                post_id = elem.attrib['PostId']
                post_id_list = postType[post_id]
                if(len(post_id_list)==2):   #checking whether it is a question or answer. If it is an answer, we need to get the parent ID.
                    post_id = post_id_list[1]
                El_id = '"'+elem.attrib['PostId']+'"' #This is to mark the answers
                
                file_path = name+"/Post"+str(post_id)+".knolml" #creating the path for KMNL file
                
                #if(post_id=='3'):
                #print(El_id)
                '''
                if(count==1):
                    break
                count+=1
                '''
                commentDate = elem.attrib['CreationDate']
                
                instance_list = []
                parentId = ''
                if(os.path.exists(file_path)):  #If KMNL file has already been created then read that file
                    
                    dict_answers = {}
                    with open(file_path) as file:
                        for sent in file:
                            if('<Instance' in sent):
                                if('OriginalId' in sent):
                                    sent = sent.split(' ')
                                    #print(sent[4],post_id)
                                    dict_answers[sent[4]] = sent[4]
    
                                    if(dict_answers.get(El_id)!=None):
                                        
                                        parentId = dict_answers[El_id]
                                        '''
                                        if(post_id=='22'):
                                            print(parentId)
                                        '''
                                    sent = " ".join(sent)
                                else:
                                    if(dict_answers.get(El_id)==None):
                                        parentId = '"1"'
                                    
    
                                    
                                #print(parentId)
                            instance_list.append(sent)
    
                    '''
                    if(post_id=='22'):
                        print(dict_answers)
                        print(El_id,parentId)
                    '''
                    
                                    
                    
                    
                    open(file_path, "w").close()
        
    
                comment_index = 0
                index_append_list = 0
                flag = 0
                
                for sent in instance_list:
                    if(flag==1): #This returns that a hit has been found where the comment should be appended 
                        break
                    if("<CreationDate>" in sent):                    
                        post_date = sent.replace('<CreationDate>','')
                        post_date = post_date.replace('</CreationDate>','')
                        post_date = post_date.replace('\t','')
                        post_date = post_date.replace('\n','')
                        post_date = post_date.replace(' ','')
                        post_date = datetime.strptime(post_date, date_format)
                        comDate = datetime.strptime(commentDate, date_format)
                        
                        '''
                        This part finds the index where this comment should be put
                        '''
                        if(comDate<post_date):                        
                            for new_sent in range(comment_index,0,-1):
                                if('<Instance' in instance_list[new_sent]):
                                    index_append_list = new_sent
                                    
                                    flag = 1
                                    break
    
                        
                    comment_index+=1
                
                if(index_append_list==0):
                    index_append_list = len(instance_list)+1
                
                
                '''
                This part adds the comment to its respective place in the list
                '''
                t = '\t'
                Instance = t+t+"<Instance Id= "+"zzz"+" "+"ParentId= "+parentId+" InstanceType="+'"'+"Comments"+'" >\n'
                instance_list.insert(index_append_list,Instance)
                index_append_list+=1
                TimeStamp = t+t+t+"<TimeStamp>\n "
                instance_list.insert(index_append_list,TimeStamp)
                index_append_list+=1
                if(elem.attrib.get("CreationDate") != None):
                    CreationDate = t+t+t+t+"<CreationDate>"+elem.attrib['CreationDate']+"</CreationDate> \n"
                    #myFile.write(CreationDate)
                    instance_list.insert(index_append_list,CreationDate)
                    index_append_list+=1
                TimeStamp = t+t+t+"</TimeStamp>\n "
                #myFile.write(TimeStamp)
                instance_list.insert(index_append_list,TimeStamp)
                index_append_list+=1
                
                contributors = t+t+t+"<Contributors>\n "
                instance_list.insert(index_append_list,contributors)
                index_append_list+=1
                
                if(elem.attrib.get('UserId')!=None):
                    ownerUserId = t+t+t+t+"<OwnerUserId>"+elem.attrib['UserId']+"</OwnerUserId> \n"
                    instance_list.insert(index_append_list,ownerUserId)
                    index_append_list+=1
                else:
                    ownerUserId = t+t+t+t+"<OwnerUserId>"+str(0)+"</OwnerUserId> \n"
                    instance_list.insert(index_append_list,ownerUserId)
                    index_append_list+=1
                    
                contributors = t+t+t+"</Contributors>\n "
                instance_list.insert(index_append_list,contributors)
                index_append_list+=1
                    
                if(elem.attrib.get('Text')!=None):        
                    Body = t+t+t+"<Body>\n"
                    instance_list.insert(index_append_list,Body)
                    index_append_list+=1
                    text_field = t+t+t+t+"<Text Type= "+'"'+"text"+'"'+" Bytes="+'"'+str(len(elem.attrib['Text']))+'">\n'
                    instance_list.insert(index_append_list,text_field)
                    index_append_list+=1
        
                    text_body = textwrap.indent(text=elem.attrib['Text'], prefix=t+t+t+t+t)
                    text_body = html.escape(text_body)
                    Body_text = text_body+"\n"
                    instance_list.insert(index_append_list,Body_text)
                    index_append_list+=1
                    text_field = t+t+t+t+"</Text>\n"
                    instance_list.insert(index_append_list,text_field)
                    index_append_list+=1
                    Body = t+t+t+"</Body>\n"
                    instance_list.insert(index_append_list,Body)
                    index_append_list+=1

                Reputation_tag = t+t+t+"<Credit> \n"
                instance_list.insert(index_append_list,Reputation_tag)
                index_append_list+=1
                if(elem.attrib.get("Score") != None):
                    score = t+t+t+t+"<Score>"+elem.attrib['Score']+"</Score>\n"
                    instance_list.insert(index_append_list,score)
                    index_append_list+=1
                    
            
                Reputation_tag = t+t+t+"</Credit> \n"
                instance_list.insert(index_append_list,Reputation_tag)
                index_append_list+=1                       

                
                Instance = t+t+"</Instance>\n"
                instance_list.insert(index_append_list,Instance)
                
                '''
                Writing everything to the file
                '''
                open(file_path, "w").close()
                
                '''
                instanceId = 1
                dict_answers = {'1':'1'}
                
                for myLine in instance_list:
                    if('<Instance Id' in myLine):
                        myLine = myLine.split(' ')
                        myLine[2] = '"'+str(instanceId)+'"'
                        myLine = " ".join(myLine)
                        
                        if(myLine[3] == 'OriginalId='):   
                            if(dict_answers.get(myLine[4])==None):
                                dict_answers[myLine[4]] = str(instanceId)
                        
                        instanceId+=1
                '''
                
                
                instanceId = 1
                '''            
                if(post_id == '22'):
                    print(dict_answers)            
                '''
                
                dict_answers = {'"1"':'"1"'}
                #print(dict_answers)
                with open(file_path,'a') as myFile:
                    
                    for myLine in instance_list:
                        '''
                        if(post_id=='22'):
                            if('<Instance Id' in myLine):
                                print(myLine)
                        '''
                        if('<Instance Id' in myLine):
    
                            myLine = myLine.split(' ')
                            myLine[2] = '"'+str(instanceId)+'"'
                            #myLine[2] = '"'+str(instanceId)+'"'
                            
                            
                            if(myLine[3] == 'OriginalId='):   
                                if(dict_answers.get(myLine[4])==None):
                                    dict_answers[myLine[4]] = '"'+str(instanceId)+'"'
    
                            
                            
                            if(myLine[3]=='ParentId='):
                                
                                #print(file_path)
                                #print(myLine[0],myLine[1],myLine[2],myLine[3],myLine[4],myLine[5],myLine[6])
                                
                                '''
                                if(post_id == '22'):
                                    print(myLine[5])
                                '''
                                #print(file_path)
                                #print(myLine)
                                
                                if((dict_answers.get(myLine[4])!=None)):
                                    '''
                                    if(post_id == '21'):
                                        print(myLine[4])
                                    '''
                                    myLine[4] = dict_answers[myLine[4]]
                                    
                            
                            
                            myLine = " ".join(myLine)
                            instanceId+=1
                            
                            '''
                            if(post_id == '22'):
                                print(dict_answers)
                            '''
                        myFile.write(myLine)    
    
                elem.clear()
                root_posts.clear()

        file_list = glob.glob(name+"/*.knolml")
        for file_path in file_list:
            if(os.path.isfile(file_path)):
                with open(file_path,"a",encoding='utf-8') as myFile:
                    myFile.write("\t</KnowledgeData>\n")
                    myFile.write("</KnolML>\n")                 
                
    
    @staticmethod            
    def convert(name, *args, **kwargs):
        name = name.lower()
        if(kwargs.get('download')!=None):             
            down = kwargs['download']
            if(down):

                
                with open('stackExchangeList.txt','r') as myFile:
                    for line in myFile:
                        if(name in line):
                            siteName = line[:-1]
                print("Downloading the "+name+" Stack Exchange data dump")
                download('stackexchange', verbose=True, glob_pattern=siteName)
                qaConverter.make_path(name)
                call(["7z","x",'stackexchange/'+siteName,'-o'+name])
                
                '''
                calling the KML converter for Stack_exchange
                '''
                
                if(kwargs.get('posthistory')!=None):
                    ph = kwargs['posthistory']
                    if(ph):
                        print("Converting PostHistory of "+name+" Stack Exchange into knolml")
                        qaConverter.make_path(name+"/PostHistory")
                        namePh = name+"/PostHistory"
                        qaConverter.commentsConversion(namePh,0)
                        print("PostHistory conversion completed for "+name+" Stack Exchange")
                if(kwargs.get('post')!=None):
                    p = kwargs['post']
                    if(p):
                        print("Converting Posts of "+name+" Stack Exchange into knolml")
                        qaConverter.make_path(name+"/Posts")
                        nameP = name+"/Posts"
                        qaConverter.commentsConversion(nameP,1)
                        print("Posts conversion completed for "+name+" Stack Exchange")
                
            else:
                '''
                calling the KML converter for Stack_exchange
                '''
                with open('stackExchangeList.txt','r') as myFile:
                    for line in myFile:
                        if(name in line):
                            siteName = line[:-1]

                call(["7z","x",'stackexchange/'+siteName,'-o'+name])
                
                if(kwargs.get('posthistory')!=None):
                    ph = kwargs['posthistory']
                    if(ph):
                        print("Converting PostHistory of "+name+" Stack Exchange into knolml")
                        qaConverter.make_path(name+"/PostHistory")
                        namePh = name+"/PostHistory"
                        qaConverter.commentsConversion(namePh,0)
                        print("PostHistory conversion completed for "+name+" Stack Exchange")

                if(kwargs.get('post')!=None):
                    p = kwargs['post']
                    if(p):
                        print("Converting Posts of "+name+" Stack Exchange into knolml")
                        qaConverter.make_path(name+"/Posts")
                        nameP = name+"/Posts"
                        qaConverter.commentsConversion(nameP,1)
                        print("Posts conversion completed for "+name+" Stack Exchange")
                


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct  8 15:07:50 2019

@author: descentis
"""

#from sampleExtractdb import display_data
import requests

class wikiExtract(object):
    
    def get_articles_by_category(self, category_name):
        '''
        this definition can be used to get the names of article
        related to a category
        '''
        url1 = 'https://en.wikipedia.org/w/api.php?action=query&list=categorymembers&format=json&cmtitle=Category:'
        category_dict = {}
        url = url1+category_name
        article_list = []
        extra_category = []
        while(True):
            r = requests.get(url)
            try:
                data = r.json()
            except:
                break
            pages = data['query']['categorymembers']
            for i in pages:
                
                if 'Category:' in i['title']:
                    extra_category.append(i)
                elif 'Template:' not in i['title']:
                    article_list.append(i)
            
            if data.get('continue')!=None:
                url = url+'&cmcontinue='+data['continue']['cmcontinue']
            else:
                break
        category_dict[category_name] = article_list
        category_dict['extra#@#category'] = extra_category
        
        return category_dict
    
    def get_articles_by_template(self, template_name):
        '''
        this definition can be used to get the names of article
        based on category
        '''
        url1 = 'https://en.wikipedia.org/w/api.php?action=query&list=embeddedin&eilimit=5&format=json&eititle=Template:'
        template_dict = {}
        url = url1+template_name
        article_list = []
        while(True):
            r = requests.get(url)
            data = r.json()
            pages = data['query']['embeddedin']
            for i in pages:
                article_list.append(i)
            
            if data.get('continue')!=None:
                url = url+'&eicontinue='+data['continue']['eicontinue']
            else:
                break
        template_dict[template_name] = article_list
        
        return template_dict

'''
w = wikiExtract()
B = w.get_articles_by_template(['Black Lives Matter'])
for key,val in B.items():
    print(key, val)
'''



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jun 29 11:35:05 2019

@author: descentis
"""

from SPARQLWrapper import SPARQLWrapper, JSON
import pandas as pd
sparql = SPARQLWrapper("https://query.wikidata.org/bigdata/namespace/wdq/sparql")
sparql.setQuery("""
SELECT ?sitelink ?itemLabel  WHERE {
    ?sitelink schema:about ?item;
    wikibase:badge wd:Q17437796. # Sitelink is badged as a Featured Article  
    ?sitelink schema:isPartOf <https://en.wikipedia.org/>.
    SERVICE wikibase:label { bd:serviceParam wikibase:language "[AUTO_LANGUAGE],en" } .
}
""")
sparql.setReturnFormat(JSON)
results = sparql.query().convert()

names = []
for item in results['results']['bindings']:
    names.append({
        'Site Link': item['sitelink']['value'],
        'Label': item['itemLabel']['value']})

df = pd.DataFrame(names)
print(len(df))
df



import difflib
import xml.etree.ElementTree as ET
import math
import os

def is_number(s):
    try:
        int(s)
        return True
    except ValueError:
        return False

def encode(str1, str2):
	output = ""
	s = [x.replace("\n", "`").replace("-", "^") for x in str1.split(" ")]

	s2 = [x.replace("\n", "`").replace("-", "^") for x in str2.split(" ")]

	i = 0
	while(True):
		if i == len(s):
			break;
		if s[i].isspace() or s[i] == '':
			del s[i]
		else:	
			i += 1	
	i = 0
	while(True):
		if i == len(s2):
			break;
		if s2[i].isspace() or s2[i] == '':
			del s2[i]
		else:	
			i += 1	
			
	d = difflib.Differ()

	result = list(d.compare(s, s2))

	pos = 0
	neg = 0

	for x in result:
		if x[0] == " ":
			pos += 1
			if neg != 0:
				output += "-"+str(neg)+" "
				neg = 0
		elif x[0] == "-":
			neg += 1
			if pos != 0:
				output += str(pos)+" "
				pos = 0	
		elif x[0] != "?":
			if pos != 0:
				output += str(pos)+" "
				pos = 0	
			if neg != 0:
				output += "-"+str(neg)+" "
				neg = 0
			if is_number(x[2:]):
				output += "'"+x[2:]+"' "
			else:			
				output += x[2:]+" "
	if pos != 0:
		output += str(pos)+" "
	if neg != 0:
		output += "-"+str(neg)+" "
	return output.replace("\t\t\t", "")

#Main function

def compress(file_name, directory):
	# file_name = input("Enter path of KML file:")

    tree = ET.parse(file_name)
    r = tree.getroot()
    for child in r:
        if('KnowledgeData' in child.tag):
            root = child
            
    last_rev = ""
    length = len(root.findall('Instance'))

    print(length, "revisions found")

    count = 0
    intervalLength =  int((math.log(length)) ** 2);  

    # Keep the Orginal text after every 'm' revisions
    m = intervalLength+1
    for each in root.iter('Text'):
        count += 1
        if m != intervalLength+1:
            current_str = each.text
            each.text = encode(prev_str, current_str)
            prev_str = current_str
            # print("Revision ", count, " written")
			
            m = m - 1
            if m == 0:
                m = intervalLength+1
        else:
            prev_str = each.text
            # print("Revision ", count, " written")
            m = m - 1
            continue

    print("KNML file created")
	
    # Creating directory 
    if not os.path.exists(directory):
        os.mkdir(directory)

    # Changing file path to include directory
    file_name = file_name.split('/')
    file_name = directory+'/'+file_name[-1]
    '''
    file_name.insert(-1, directory)
    separator = '/'
    file_name = separator.join(file_name)
    '''

    tree.write(file_name[:-7]+'Compressed.knolml')
    f = open(file_name[:-7]+'Compressed.knolml')
    f_str = f.read()
    f.close()

    f2 = open(file_name[:-7]+'Compressed.knolml', "w")
    f2.write("<?xml version='1.0' encoding='utf-8'?>\n"+f_str)
    f2.close()
